/* monomix.c

 This file has poor memory protection. Failures during malloc() will
 not recover nicely.
 */

/*****************************************************************************/

#include <config.h>

#include <stdlib.h>
#include <string.h>

/*****************************************************************************/

#include "ladspa.h"

void __attribute__ ((constructor)) initMonomix(void);
void __attribute__ ((destructor)) finiMonomix(void);

/*****************************************************************************/

/* The port numbers for the plugin: */

#define MM_CTL_MODE 0
#define MM_CTL_AMOUNT_L 1
#define MM_CTL_AMOUNT_R 2
#define MM_INPUT_L 3
#define MM_INPUT_R 4
#define MM_OUTPUT_L 5
#define MM_OUTPUT_R 6

#define MM_PORTCOUNT 7

/* The modes of operation */

#define MM_MODE_L_PLUS_R 0
#define MM_MODE_L_MINUS_R 1
#define MM_MODE_L 2
#define MM_MODE_R 3
#define MM_MODE_CUSTOM 4

/*****************************************************************************/

/* The structure used to hold port connection information and state
 (actually gain controls require no further state). */

typedef struct {

	/* Ports:
	 ------ */

	LADSPA_Data * m_pfControlMode;
	LADSPA_Data * m_pfControlAmountL;
	LADSPA_Data * m_pfControlAmountR;
	LADSPA_Data * m_pfInputBufferL;
	LADSPA_Data * m_pfInputBufferR;
	LADSPA_Data * m_pfOutputBufferL;
	LADSPA_Data * m_pfOutputBufferR;

} Monomix;

/*****************************************************************************/

/* Construct a new plugin instance. */
LADSPA_Handle instantiateMonomix(const LADSPA_Descriptor * Descriptor,
		unsigned long SampleRate) {
	return malloc(sizeof(Monomix));
}

/*****************************************************************************/

/* Connect a port to a data location. */
void connectPortToMonomix(LADSPA_Handle Instance, unsigned long Port,
		LADSPA_Data * DataLocation) {

	Monomix * psMonomix;

	psMonomix = (Monomix *) Instance;
	switch (Port) {
	case MM_CTL_MODE:
		psMonomix->m_pfControlMode = DataLocation;
		break;
	case MM_CTL_AMOUNT_L:
		psMonomix->m_pfControlAmountL = DataLocation;
		break;
	case MM_CTL_AMOUNT_R:
		psMonomix->m_pfControlAmountR = DataLocation;
		break;
	case MM_INPUT_L:
		psMonomix->m_pfInputBufferL = DataLocation;
		break;
	case MM_INPUT_R:
		psMonomix->m_pfInputBufferR = DataLocation;
		break;
	case MM_OUTPUT_L:
		psMonomix->m_pfOutputBufferL = DataLocation;
		break;
	case MM_OUTPUT_R:
		psMonomix->m_pfOutputBufferR = DataLocation;
		break;
	}
}

/*****************************************************************************/

void runMonomix(LADSPA_Handle Instance, unsigned long SampleCount) {

	LADSPA_Data * pfInputL;
	LADSPA_Data * pfInputR;
	LADSPA_Data * pfOutputL;
	LADSPA_Data * pfOutputR;
	LADSPA_Data amountL;
	LADSPA_Data amountR;
	Monomix * psMonomix;
	unsigned long lSampleIndex;

	psMonomix = (Monomix *) Instance;
	pfOutputL = psMonomix->m_pfOutputBufferL;

	switch ((int) *(psMonomix->m_pfControlMode)) {
	case MM_MODE_L_PLUS_R:
		pfInputL = psMonomix->m_pfInputBufferL;
		pfInputR = psMonomix->m_pfInputBufferR;
		for (lSampleIndex = 0; lSampleIndex < SampleCount; lSampleIndex++)
			*(pfOutputL++) = 0.5 * (*(pfInputL++) + *(pfInputR++));
		break;
	case MM_MODE_L_MINUS_R:
		pfInputL = psMonomix->m_pfInputBufferL;
		pfInputR = psMonomix->m_pfInputBufferR;
		for (lSampleIndex = 0; lSampleIndex < SampleCount; lSampleIndex++)
			*(pfOutputL++) = 0.5 * (*(pfInputL++) - *(pfInputR++));
		break;
	case MM_MODE_L:
		pfInputL = psMonomix->m_pfInputBufferL;
		for (lSampleIndex = 0; lSampleIndex < SampleCount; lSampleIndex++)
			*(pfOutputL++) = *(pfInputL++);
		break;
	case MM_MODE_R:
		pfInputR = psMonomix->m_pfInputBufferR;
		for (lSampleIndex = 0; lSampleIndex < SampleCount; lSampleIndex++)
			*(pfOutputL++) = *(pfInputR++);
		break;
	case MM_MODE_CUSTOM:
		pfInputL = psMonomix->m_pfInputBufferL;
		pfInputR = psMonomix->m_pfInputBufferR;
		amountL = *(psMonomix->m_pfControlAmountL);
		amountR = *(psMonomix->m_pfControlAmountR);
		for (lSampleIndex = 0; lSampleIndex < SampleCount; lSampleIndex++)
			*(pfOutputL++) = amountL * *(pfInputL++) + amountR * *(pfInputR++);
		break;
	}
	pfOutputL = psMonomix->m_pfOutputBufferL;
	pfOutputR = psMonomix->m_pfOutputBufferR;
	for (lSampleIndex = 0; lSampleIndex < SampleCount; lSampleIndex++)
		*(pfOutputR++) = *(pfOutputL++);
}

/*****************************************************************************/

void cleanupMonomix(LADSPA_Handle Instance) {
	free(Instance);
}

/*****************************************************************************/

LADSPA_Descriptor * g_psDescriptor = NULL;

/*****************************************************************************/

/* _init() is called automatically when the plugin library is first
 loaded. */
void initMonomix() {

	char ** pcPortNames;
	LADSPA_PortDescriptor * piPortDescriptors;
	LADSPA_PortRangeHint * psPortRangeHints;

	g_psDescriptor = (LADSPA_Descriptor *) malloc(sizeof(LADSPA_Descriptor));

	if (g_psDescriptor) {

		g_psDescriptor->UniqueID = UID_MONOMIX;
		g_psDescriptor->Label = strdup("monomix");
		g_psDescriptor->Properties = LADSPA_PROPERTY_HARD_RT_CAPABLE;
		g_psDescriptor->Name = strdup("Mono Mixer");
		g_psDescriptor->Maker = strdup("Luis Garrido");
		g_psDescriptor->Copyright = strdup("GPL");
		g_psDescriptor->PortCount = MM_PORTCOUNT;
		piPortDescriptors = (LADSPA_PortDescriptor *) calloc(MM_PORTCOUNT,
				sizeof(LADSPA_PortDescriptor));
		g_psDescriptor->PortDescriptors
				= (const LADSPA_PortDescriptor *) piPortDescriptors;
		piPortDescriptors[MM_CTL_MODE] = LADSPA_PORT_INPUT
				| LADSPA_PORT_CONTROL;
		piPortDescriptors[MM_CTL_AMOUNT_L] = LADSPA_PORT_INPUT
				| LADSPA_PORT_CONTROL;
		piPortDescriptors[MM_CTL_AMOUNT_R] = LADSPA_PORT_INPUT
				| LADSPA_PORT_CONTROL;
		piPortDescriptors[MM_INPUT_L] = LADSPA_PORT_INPUT | LADSPA_PORT_AUDIO;
		piPortDescriptors[MM_INPUT_R] = LADSPA_PORT_INPUT | LADSPA_PORT_AUDIO;
		piPortDescriptors[MM_OUTPUT_L] = LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO;
		piPortDescriptors[MM_OUTPUT_R] = LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO;
		pcPortNames = (char **) calloc(MM_PORTCOUNT, sizeof(char *));
		g_psDescriptor->PortNames = (const char **) pcPortNames;
		pcPortNames[MM_CTL_MODE] = strdup("Mode");
		pcPortNames[MM_CTL_AMOUNT_L] = strdup("Amount L");
		pcPortNames[MM_CTL_AMOUNT_R] = strdup("Amount R");
		pcPortNames[MM_INPUT_L] = strdup("Input L");
		pcPortNames[MM_INPUT_R] = strdup("Input R");
		pcPortNames[MM_OUTPUT_L] = strdup("Output L");
		pcPortNames[MM_OUTPUT_R] = strdup("Output R");
		psPortRangeHints = ((LADSPA_PortRangeHint *) calloc(MM_PORTCOUNT,
				sizeof(LADSPA_PortRangeHint)));
		g_psDescriptor->PortRangeHints
				= (const LADSPA_PortRangeHint *) psPortRangeHints;
		psPortRangeHints[MM_CTL_MODE].HintDescriptor
				= (LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE
						| LADSPA_HINT_INTEGER | LADSPA_HINT_DEFAULT_0);
		psPortRangeHints[MM_CTL_MODE].LowerBound = MM_MODE_L_PLUS_R;
		psPortRangeHints[MM_CTL_MODE].UpperBound = MM_MODE_CUSTOM;
		psPortRangeHints[MM_CTL_AMOUNT_L].HintDescriptor
				= (LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE
						| LADSPA_HINT_DEFAULT_MAXIMUM);
		psPortRangeHints[MM_CTL_AMOUNT_L].LowerBound = -0.5;
		psPortRangeHints[MM_CTL_AMOUNT_L].UpperBound = 0.5;
		psPortRangeHints[MM_CTL_AMOUNT_R].HintDescriptor
				= (LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE
						| LADSPA_HINT_DEFAULT_MAXIMUM);
		psPortRangeHints[MM_CTL_AMOUNT_R].LowerBound = -0.5;
		psPortRangeHints[MM_CTL_AMOUNT_R].UpperBound = 0.5;
		psPortRangeHints[MM_INPUT_L].HintDescriptor = 0;
		psPortRangeHints[MM_INPUT_R].HintDescriptor = 0;
		psPortRangeHints[MM_OUTPUT_L].HintDescriptor = 0;
		psPortRangeHints[MM_OUTPUT_R].HintDescriptor = 0;
		g_psDescriptor->instantiate = instantiateMonomix;
		g_psDescriptor->connect_port = connectPortToMonomix;
		g_psDescriptor->activate = NULL;
		g_psDescriptor->run = runMonomix;
		g_psDescriptor->run_adding = NULL;
		g_psDescriptor->set_run_adding_gain = NULL;
		g_psDescriptor->deactivate = NULL;
		g_psDescriptor->cleanup = cleanupMonomix;
	}

}

/*****************************************************************************/

void deleteDescriptor(LADSPA_Descriptor * psDescriptor) {
	unsigned long lIndex;
	if (psDescriptor) {
		free((char *) psDescriptor->Label);
		free((char *) psDescriptor->Name);
		free((char *) psDescriptor->Maker);
		free((char *) psDescriptor->Copyright);
		free((LADSPA_PortDescriptor *) psDescriptor->PortDescriptors);
		for (lIndex = 0; lIndex < psDescriptor->PortCount; lIndex++)
			free((char *) (psDescriptor->PortNames[lIndex]));
		free((char **) psDescriptor->PortNames);
		free((LADSPA_PortRangeHint *) psDescriptor->PortRangeHints);
		free(psDescriptor);
	}
}

/*****************************************************************************/

/* _fini() is called automatically when the library is unloaded. */
void finiMonomix() {
	deleteDescriptor(g_psDescriptor);
}

/*****************************************************************************/

/* Return a descriptor of the requested plugin type. There are two
 plugin types available in this library (mono and stereo). */
const LADSPA_Descriptor *
ladspa_descriptor(unsigned long Index) {
	/* Return the requested descriptor or null if the index is out of
	 range. */
	switch (Index) {
	case 0:
		return g_psDescriptor;
	default:
		return NULL;
	}
}

/*****************************************************************************/

/* EOF */
